--[[
  Based on an idea of:
  
  Seb Lee-Delisle
  Blog : www.sebleedelisle.com
  Company : www.pluginmedia.net
   
  This work is licensed under a Creative Commons  2.0 License.
 
  Full details at 
  http:--creativecommons.org/licenses/by/2.0/uk/
  
  Modified by Bill Rigas
  sort.coder@gmail.com
]]

-- Base class
particleClass = gideros.class(Sprite)

-- Derived classes
ccSnowParticle = gideros.class(particleClass)
ccExplosionParticle = gideros.class(particleClass)

function ccSnowParticle:init(ptable, start)
  self.particleTable = ptable
  
  self.totalParticles = self.particleTable["maxParticles"]
  self.duration = ptable["duration"]
  
  self.emissionRate = self.totalParticles / ptable["particleLifespan"]
  self.imgTexture = Texture.new("fire.png")
  
  if start ~= nil then
    if not start then self:stop() end
  end
end

function ccExplosionParticle:init(ptable, start)
  self.particleTable = ptable
  
  self.totalParticles = self.particleTable["maxParticles"]
  self.duration = ptable["duration"]
  
  self.emissionRate = self.totalParticles / self.duration
  self.imgTexture = Texture.new("fire.png")
  
  if start ~= nil then
    if not start then self:stop() end
  end
end

function particleClass:init()
  self.totalParticles = 250		-- number of particles
 
  self.particles = {}	-- particle table
  
  self.emissionRate = 0
  self.imgTexture = nil
  self.emitCounter = 0
  self.elapsed = 0
  self.active = true
  self.particleTable = nil
  self.duration = -1
  self.particleCount = 0
end

function particleClass:postInit()
  for i=1, self.totalParticles do
    local shape = Bitmap.new(self.imgTexture)
	
	local particle = cParticle.new(shape, self.particleTable)
	table.insert(self.particles, particle)
	
	shape:setVisible(false)
	self:addChild(shape)
  end
  
  self.particleCount = self.totalParticles
  
  self:addEventListener(Event.ENTER_FRAME, self.enterFrameHandler, self)
end

function particleClass:start()
  self.active = true
end

function particleClass:stop()
  self.active = false
end

function particleClass:isStopped()
  return not self.active
end

function particleClass:enterFrameHandler(event)
  local dt = event.deltaTime
  self:updateParticles(dt)
end

function particleClass:restart()
  self.active = true
  self.elapsed = 0
  self.particleCount = 0
end

-- update existing particles
function particleClass:updateParticles(dt)
  if self.active and self.emissionRate then
    local rate = 1 / self.emissionRate
	
	if self.particleCount < self.totalParticles then
	  self.emitCounter = self.emitCounter + dt
	end
 
    while self.particleCount < self.totalParticles and self.emitCounter > rate do
		self:addParticle()
		self.emitCounter = self.emitCounter - rate
	end

	self.elapsed = self.elapsed + dt;

	if self.duration ~= -1 and self.duration < self.elapsed then
		self:stopSystem()
	end
  end
  
  local particleIdx = 1
  
  while particleIdx < self.particleCount do
    local particle = self.particles[particleIdx]
	particle:update(dt)
	
	if particle.timeToLive > 0 then
	  particleIdx = particleIdx + 1
	else
	  particle:disable()
	  
	  -- swap particles if pool is not empty
	  if particleIdx ~= self.particleCount then
	    self.particles[particleIdx] = self.particles[self.particleCount]
		self.particles[self.particleCount] = particle
		self.particleCount = self.particleCount - 1
	  end
	end
  end -- of while
end

function particleClass:stopSystem()
  self.active = false
  self.elapsed = self.duration
  self.emitCounter = 0
end

function particleClass:addParticle()
  if self.particleCount == self.totalParticles then return end
  self.particleCount = self.particleCount + 1
  
  local particle = self.particles[self.particleCount]  
  particle:reset()
end
